import {JSONResponse, TransformData} from "./interface";
import {ManagerActivity, ManagerEntity, PageProps, TimeScore, UserEntity} from "./entity";

/**
 * 空body响应
 */
export enum SimpleMessage {
    //操作失败
    ok="ok",
    //操作成功
    fail="fail"
}

/**
 * 空body响应
 */
export class EmptyBodyTransform extends TransformData<SimpleMessage, JSONResponse<SimpleMessage>>{
    protected newObject(): JSONResponse<SimpleMessage> {
        return new JSONResponse<SimpleMessage>();
    }
}

export abstract class EmptyBodyDiffMsg<T> extends TransformData<T, JSONResponse<T>>{}

export enum LoginMessage{
    ok='ok',
    fail='fail',
    form_fail='form_fail',
    valid_fail='valid_fail'
}

export class LoginRes extends JSONResponse<LoginMessage>{
    managerId?:string
}

export class LoginTransform extends TransformData<LoginMessage,LoginRes>{
    protected newObject(): LoginRes {
        return new LoginRes();
    }


    protected transformBody(data: any) {
        if(data.body) {
            this.target.managerId = data.body.managerId
        }
    }
}

export enum RegisterRes {
    //注册成功
    ok='ok',
    //系统异常，注册失败
    fail='fail',
    //用户id重复
    user_repeat='user_repeat',
    //表单解析错误
    form_parse_error='form_parse_error'
}

export class RegisterTransform extends EmptyBodyDiffMsg<RegisterRes>{
    protected newObject(): JSONResponse<RegisterRes> {
        return new JSONResponse<RegisterRes>();
    }
}


/**
 * 分页数据转换
 */
export enum PageDataMessage {
    ok="ok",
    fail="fail",
    //空数据
    empty="empty"
}

export abstract class PageDataRes extends JSONResponse<PageDataMessage>{
    dataList?:any;
    page?:PageProps;
}

export abstract class PageDataTransform<T extends PageDataRes> extends TransformData<PageDataMessage, T>{

    protected transformBody(data: any) {
        if(data.body!==null){
            this.target.dataList=data.body.body
            this.target.page={
                currentPage:data.body.currentPage,
                totalPage:data.body.totalPage,
                pageSize:data.body.pageSize
            }
        }

    }
}



export class ManagerList extends PageDataRes{
    dataList?:Array<ManagerEntity>
}

export class ManagerTransform extends PageDataTransform<ManagerList>{
    protected newObject(): ManagerList {
        return new ManagerList();
    }
}

/**
 * 用户信息实体
 */
export class FindUserRes extends PageDataRes{
    dataList?:Array<UserEntity>
}

/**
 * 用户数据转换
 */
export class FindUserTransform extends PageDataTransform<FindUserRes>{
    protected newObject(): FindUserRes {
        return new FindUserRes();
    }
}

export class FindActivityRes extends PageDataRes{
    dataList?:Array<ManagerActivity>
}

export class FindActivityTransform extends PageDataTransform<FindActivityRes>{
    protected newObject(): FindActivityRes {
        return new FindActivityRes();
    }

}

export class FindTimeScoreRes extends JSONResponse<SimpleMessage>{
    timeScore?:TimeScore;
}

export class ScoreTransform extends TransformData<SimpleMessage, FindTimeScoreRes>{
    protected newObject(): FindTimeScoreRes {
        return new FindTimeScoreRes();
    }


    protected transformBody(data: any) {
        this.target.timeScore=data.body
    }
}
